﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Data.OleDb;

namespace Silverlightdata.Web
{
    public class datamanager
    {
        public static List<photoitem> GetPhotoItems()
        {
            List<photoitem> theList = new List<photoitem>();


            //connect to the database
            OleDbConnection conn;
            OleDbDataReader dr;

            try
            {

                conn = GetConnection();
                string sql = "SELECT ID,subject,datetaken,notes,imagepath from images ORDER BY subject";
                OleDbCommand cmd = new OleDbCommand(sql);
                cmd.Connection = conn;
                conn.Open();

                dr = cmd.ExecuteReader();

                while (dr.Read())
                {
                    photoitem pi = new photoitem();
                    pi.ID = dr.GetInt32(0);
                    pi.Subject = dr.GetString(1);
                    pi.DateTaken = dr.GetDateTime(2);
                    pi.Notes = dr.GetString(3);
                    pi.ImagePath = dr.GetString(4);
                    theList.Add(pi);

                }

                conn.Close();

                conn.Dispose();

            }
            catch (Exception e)
            {
                System.Diagnostics.Debug.WriteLine(e.Message);
                photoitem pi = new photoitem();
                pi.ID = -999;
                pi.Subject = "Error";
                pi.DateTaken = DateTime.Now;
                pi.Notes = e.Message;
                pi.ImagePath = string.Empty;
                theList.Add(pi);
            }

            return theList;


        }

        public static int SavePhotoItem(photoitem pi)
        {
            if (HttpContext.Current.User == null)
            {
                return -1;
            }

            if (!(HttpContext.Current.User.Identity.IsAuthenticated))
            {
                return -1;
            }

            if (!(HttpContext.Current.User.Identity.Name == "admin"))
            {
                return -1;
            }


            if (pi.Subject == string.Empty)
            {
                return -1;
            }

            bool bNew = (pi.ID < 0);


            int returnid = -1;

            try
            {

                OleDbConnection conn;

                conn = GetConnection();
                string sql;

                if (bNew)
                {
                    //new user
                    sql = "INSERT INTO images (Subject,DateTaken,ImagePath,Notes) VALUES (@subject,@datetaken,@imagepath,@notes)";
                }
                else
                {
                    //existing user
                    sql = "UPDATE images set Subject = @Subject,DateTaken = @datetaken,ImagePath = @imagepath,Notes = @notes WHERE ID = @id";

                }

                OleDbCommand cmd = new OleDbCommand(sql);

                cmd.Parameters.Add(new OleDbParameter("@subject", pi.Subject));
                cmd.Parameters.Add(new OleDbParameter("@datetaken", pi.DateTaken.ToShortDateString()));
                cmd.Parameters.Add(new OleDbParameter("@imagepath", pi.ImagePath));
                cmd.Parameters.Add(new OleDbParameter("@notes", pi.Notes));
               
                
                if (!bNew)
                {
                    cmd.Parameters.Add(new OleDbParameter("@id", pi.ID));
                }

                cmd.Connection = conn;
                conn.Open();

                cmd.ExecuteNonQuery();



                if (!bNew)
                {
                    returnid = pi.ID;
                }
                else
                {
                    //with some providers you can just do this
                    //returnid = (int)cmd.LastInsertedId;
                    
                    //get the last insert id 
                    cmd.CommandText = "SELECT @@IDENTITY"; //using identity
                    cmd.Parameters.Clear();
                    returnid = (int) cmd.ExecuteScalar(); 

                    //take note ExecuteScalar return an object, and in sqlce @@identity returns SqlDecimal
                    //SqlDecimal x = (SqlDecimal)cmd.ExecuteScalar();
                    //so my way of converting it to int is catch the value as SqlDecimal 
                    //convert that to string and then convert the string to int
                    //this way works for me
                    //id = Convert.ToInt16(x.ToString());


                    pi.ID = returnid;
                }

                conn.Close();

                conn.Dispose();

            }
            catch (Exception e)
            {
                System.Diagnostics.Debug.WriteLine(e.Message);
            }

            return returnid;


        }

        public static bool DeletePhotoItem(int id)
        {
            if (HttpContext.Current.User == null)
            {
                return false;
            }

            if (!(HttpContext.Current.User.Identity.IsAuthenticated))
            {
                return false;
            }

            if (!(HttpContext.Current.User.Identity.Name == "admin"))
            {
                return false;
            }

            OleDbConnection conn;

            conn = GetConnection();
            string sql = "delete from images where id = @ID";

            OleDbCommand cmd = new OleDbCommand(sql);
            cmd.Parameters.Add(new OleDbParameter("@ID", id));
            cmd.Connection = conn;
            conn.Open();

            cmd.ExecuteNonQuery();

            conn.Close();

            conn.Dispose();
            return true;
        }

        private static OleDbConnection GetConnection()
        {
            OleDbConnection conn;
            string datapath = System.IO.Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "App_Data");
            string s = "Provider=Microsoft.Jet.OLEDB.4.0;Data Source=" + datapath + @"\photos.mdb;User Id=admin;Password=;";
            conn = new OleDbConnection(s);

            return conn;

        }
    }
}
