﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Windows.Threading;
using System.Windows.Media.Imaging;
using System.IO;

namespace Silverlightdata
{
    public partial class EditForm : UserControl
    {
        private photodbsvc.photoitem theItem;
        public DispatcherTimer LoadThumbTimer;
        private photodbsvc.photodbSoapClient photosvc;
        private WebClient wc;
        
        public EditForm(photodbsvc.photoitem ph)
        {
            InitializeComponent();
            theItem = ph;

            photosvc = new photodbsvc.photodbSoapClient();
            wc = new WebClient();

            LoadThumbTimer = new DispatcherTimer();

            //hook up event handlers
            btnSave.Click += new RoutedEventHandler(btnSave_Click);
            btnClose.Click += new RoutedEventHandler(btnClose_Click);
            btnUpload.Click += new RoutedEventHandler(btnUpload_Click);
            photosvc.SavePhotoItemCompleted += new EventHandler<Silverlightdata.photodbsvc.SavePhotoItemCompletedEventArgs>(photosvc_SavePhotoItemCompleted);
            wc.OpenWriteCompleted += new OpenWriteCompletedEventHandler(wc_OpenWriteCompleted);
       
            LoadThumbTimer.Tick += new EventHandler(LoadThumbTimer_Tick);
            imThumb.ImageFailed += new EventHandler<ExceptionRoutedEventArgs>(imThumb_ImageFailed);

            Loaded += new RoutedEventHandler(EditForm_Loaded);
        }

        void imThumb_ImageFailed(object sender, ExceptionRoutedEventArgs e)
        {
            this.lbMessage.Text = "Cannot load image.";
        }

        void LoadThumbTimer_Tick(object sender, EventArgs e)
        {
            this.LoadThumbTimer.Stop();
            LoadThumb();
        }

        void wc_OpenWriteCompleted(object sender, OpenWriteCompletedEventArgs e)
        {
            if (e.Error != null)
            {
                this.lbMessage.Text = "Error uploading. " + e.Error.Message;
                return;
            }

            Stream data = (Stream)e.UserState;
            UploadData(data, e.Result);
            e.Result.Close();
            data.Close();
            this.lbMessage.Text = "";
            this.LoadThumbTimer.Start();
        }

        void btnUpload_Click(object sender, RoutedEventArgs e)
        {
        OpenFileDialog dlg = new OpenFileDialog();
        dlg.Multiselect = false;
        dlg.Filter = "JPEG files|*.jpg";
        if (dlg.ShowDialog() == true)
        {
            FileInfo theFile = dlg.File;
            UploadFile(theFile.Name, theFile.OpenRead());
        }
        }

        void btnClose_Click(object sender, RoutedEventArgs e)
        {
        //return to list 
        Page p  = (Page)App.Current.RootVisual;
        p.SwitchPage(new ListForm());
        }

        void photosvc_SavePhotoItemCompleted(object sender, Silverlightdata.photodbsvc.SavePhotoItemCompletedEventArgs e)
        {
             this.btnSave.IsEnabled = true;

        if (e.Error != null) {
            this.lbMessage.Text = "Error saving. " + e.Error.Message;
            return;
        }

        if (e.Result > 0)
        {
            this.theItem.ID = e.Result;
            this.lbMessage.Text = "Save completed";
        }
        else
        {
            this.lbMessage.Text = "Save failed";
        }
        }

        void btnSave_Click(object sender, RoutedEventArgs e)
        {
        //save the item back to the database
        this.btnSave.IsEnabled = false;
        photosvc.SavePhotoItemAsync(this.theItem);
        }

        void EditForm_Loaded(object sender, RoutedEventArgs e)
        {
        
        this.DataContext = theItem;
        this.LoadThumbTimer.Interval = new TimeSpan(0, 0, 1);
        this.LoadThumb();
        }

        void LoadThumb()
        {
            if ( theItem.ImagePath != String.Empty) {
            BitmapImage bm = new BitmapImage();
            Uri theUri = new Uri(theItem.ImagePath, UriKind.Relative);
            bm.UriSource = theUri;
            this.imThumb.Source = bm;
            } else {
            this.imThumb.Source = null;

            }
        }

        void UploadFile(String FileName, Stream data )
        {
        Uri u = new Uri(Application.Current.Host.Source, "../fileupload.ashx/?filename=" + FileName);
        wc.OpenWriteAsync(u, null, data);
        theItem.ImagePath = "/pix/" + FileName;
        }

        void UploadData(Stream data, Stream output)
        {
            Byte[] buffer = new Byte[4096];
            int bytesRead;
            bytesRead = data.Read(buffer, 0, buffer.Length);
            while (bytesRead != 0)
            {
                output.Write(buffer, 0, bytesRead);
                bytesRead = data.Read(buffer, 0, buffer.Length);
            }

        }
    }
}
