﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Windows.Media.Imaging;

namespace Silverlightdata
{
    public partial class ListForm : UserControl
    {
        photodbsvc.photodbSoapClient photosvc;
        securitysvc.securitySoapClient secsvc;
        string PendingOperation = string.Empty;
        IEnumerable<photodbsvc.photoitem> allPhotos;

        public ListForm()
        {
            InitializeComponent();
            PendingOperation = "Load";
            photosvc = new photodbsvc.photodbSoapClient();
            secsvc = new securitysvc.securitySoapClient();

            //set up event handlers for web services
            secsvc.GetLoginStatusCompleted += new EventHandler<Silverlightdata.securitysvc.GetLoginStatusCompletedEventArgs>(secsvc_GetLoginStatusCompleted);
            photosvc.DeletePhotoItemCompleted += new EventHandler<Silverlightdata.photodbsvc.DeletePhotoItemCompletedEventArgs>(photosvc_DeletePhotoItemCompleted);
            photosvc.GetPhotoItemsCompleted += new EventHandler<Silverlightdata.photodbsvc.GetPhotoItemsCompletedEventArgs>(photosvc_GetPhotoItemsCompleted);


            //set up event handlers for controls
            Loaded += new RoutedEventHandler(ListForm_Loaded);
            grdPhotos.SelectionChanged += new SelectionChangedEventHandler(grdPhotos_SelectionChanged);
            btnEdit.Click += new RoutedEventHandler(btnEdit_Click);
            btnDelete.Click += new RoutedEventHandler(btnDelete_Click);
            btnAdd.Click += new RoutedEventHandler(btnAdd_Click);
            btnSearch.Click += new RoutedEventHandler(btnSearch_Click);
            btnLogin.Click += new RoutedEventHandler(btnLogin_Click);
            imPhoto.ImageFailed += new EventHandler<ExceptionRoutedEventArgs>(imPhoto_ImageFailed);

        }

        void btnLogin_Click(object sender, RoutedEventArgs e)
        {
            Uri u = new Uri(Application.Current.Host.Source, "../login.aspx");
            System.Windows.Browser.HtmlPage.Window.Navigate(u);
        }

        void imPhoto_ImageFailed(object sender, ExceptionRoutedEventArgs e)
        {
            this.lbMessage.Text = "Cannot load image";
        }

        void btnSearch_Click(object sender, RoutedEventArgs e)
        {
            if (this.btnSearch.Content.ToString().Trim() == "Search")
            {

                //based on idea from http://www.mostlydevelopers.com/mostlydevelopers/blog/post/2009/02/19/Silverlight-DataGrid-ItemsSource-Filter.aspx

                string searched = this.txtSearch.Text.Trim();

                if (searched == String.Empty)
                {
                    return;
                }

                IEnumerable<photodbsvc.photoitem> source = (IEnumerable<photodbsvc.photoitem>)grdPhotos.ItemsSource;

                //save the full list
                this.allPhotos = source;

                grdPhotos.ItemsSource = source.Where((photodbsvc.photoitem p) => p.Subject.IndexOf(searched, StringComparison.CurrentCultureIgnoreCase) > -1);

                this.btnSearch.Content = "All items";
                this.txtSearch.Text = String.Empty;
            }
            else
            {

                //clear the filter
                grdPhotos.ItemsSource = allPhotos;
                this.btnSearch.Content = "Search";
            }

        }

        void btnAdd_Click(object sender, RoutedEventArgs e)
        {
            photodbsvc.photoitem pi = new photodbsvc.photoitem();
            pi.ID = -1;
            pi.DateTaken = DateTime.Now;
            pi.ImagePath = string.Empty;

            Page p = (Page)(App.Current.RootVisual);
            p.SwitchPage(new EditForm(pi));
        }

        void btnDelete_Click(object sender, RoutedEventArgs e)
        {
            if (grdPhotos.SelectedItem == null)
            {
                return;
            }

            photodbsvc.photoitem pi = (photodbsvc.photoitem)grdPhotos.SelectedItem;
            photosvc.DeletePhotoItemAsync(pi.ID);
        }

        void btnEdit_Click(object sender, RoutedEventArgs e)
        {
            if (grdPhotos.SelectedItem == null)
            {
                return;
            }

            photodbsvc.photoitem pi = (photodbsvc.photoitem)grdPhotos.SelectedItem;
            Page p = (Page)App.Current.RootVisual;
            p.SwitchPage(new EditForm(pi));
        }

        void grdPhotos_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (grdPhotos.SelectedItem == null)
            {
                this.imPhoto.Source = null;
                return;
            }

            //get the current item

            photodbsvc.photoitem pi = (photodbsvc.photoitem)grdPhotos.SelectedItem;

            if (pi.ImagePath != String.Empty)
            {

                BitmapImage bm = new BitmapImage();
                Uri theUri = new Uri(pi.ImagePath, UriKind.Relative);
                bm.UriSource = theUri;
                this.imPhoto.Source = bm;

            }
            else
            {
                this.imPhoto.Source = null;

            }

            if (pi.Notes != String.Empty)
            {
                this.txtNotes.Text = pi.Notes;
            }
            else
            {
                this.txtNotes.Text = String.Empty;
            }
        }

        void ListForm_Loaded(object sender, RoutedEventArgs e)
        {
            //get the login status
            secsvc.GetLoginStatusAsync();
        }

        void photosvc_GetPhotoItemsCompleted(object sender, Silverlightdata.photodbsvc.GetPhotoItemsCompletedEventArgs e)
        {

            if (e.Error != null)
            {
                this.lbMessage.Text = "Error retrieving data. " + e.Error.Message;
                return;
            }

            photodbsvc.ArrayOfPhotoitem photos = e.Result;

            if (photos != null)
            {
                this.grdPhotos.ItemsSource = photos;
                if (photos.Count > 0)
                {
                    this.grdPhotos.SelectedIndex = 0;
                }
            }
        }

        void photosvc_DeletePhotoItemCompleted(object sender, Silverlightdata.photodbsvc.DeletePhotoItemCompletedEventArgs e)
        {
            if (e.Error != null)
            {
                this.lbMessage.Text = "Error deleting item. " + e.Error.Message;
                return;
            }

            if (e.Result)
            {
                this.lbMessage.Text = "Item deleted";
            }
            else
            {
                this.lbMessage.Text = "Delete failed";
            }

            photosvc.GetPhotoItemsAsync();

        }

        void secsvc_GetLoginStatusCompleted(object sender, Silverlightdata.securitysvc.GetLoginStatusCompletedEventArgs e)
        {
            try
            {
                if (e.Result)
                {
                    if (PendingOperation == "Load")
                    {
                        photosvc.GetPhotoItemsAsync();
                    }
                }
                else
                {
                    this.lbMessage.Text = "You are not logged in.";
                }
            }
            finally
            {
                PendingOperation = string.Empty;
            }
        }
    }
}
